/*  FAXDFLTS.H  Header file for applications that use either of the CAS
    Toolkit functions FAXSend or FAXSubmitTask.

    Defines with initial values a default File Transfer Record and Event
    Control structure.

    This file must be included in any application source files that use
    either of these two functions.
==============================================================================*/

/*
    DEFAULTS-initialized File Transfer Record List structure:  This contains
    the default File Transfer Record used by functions FAXSend and
    FAXSubmitTask.  If the caller of these functions specifies the files to
    send, the functions get only the file names and types from the *files
    parameter; the rest of the File Transfer Record fields come from this
    structure.
*/
FTRLIST DefaultsFTRL =
    {
      {
        ASCII,     /* FileType             Type of file (ASCII, PCX, DCX) */
        NORMAL,    /* TextSize             0: 80colx66lin, 1: 132colx88lin */
        0,         /* FileStatus           Status of file (opened, moved etc) */
        0,         /* BytesSent            Bytes transmitted so far */
        0,         /* FileSize             Size of file in bytes */
        0,         /* PagesSent            Pages transmitted so far */
        0,         /* PageCount            # of pages in this file */
        "c:\\autoexec.bat",
                   /* FileName[80]         Full path and name of file */
        0,         /* AddPageIncrements    Additional 1/8 inch increments */
        0,         /* PageLength           Page length in inches */
        0          /* RESERVED[31]         Reserved for Intel use */
      },
      NULL         /* pointer to next FTRLIST element */
    };

/*
    DEFAULTS-initialized Event Control structure:  Used by function FAXSend.
    This contains default Task Control File settings, no cover page, and a
    pointer to the default File Transfer Record List defined above.
*/
ECS DefaultsECS =
    { { SEND,      /* EventType;            Type of event (send, receive etc) */
        FAX_100,   /* TransferType;         Type of transfer (file, fax etc) */
        0,         /* EventStatus;          Status of event */
        0,         /* EventTime;            Time to send or time received */
        0x0021,    /* EventDate;            DOS format for 1-1-80  */
        1,         /* FileCount;            Number of files to transfer */
        383,       /* FTROffset;            Offset of first FTR in TCF on disk */
        0,         /* Phone[47];            [0-9] and [,!PTM*#] have meaning */
        0,         /* ApplicationTag[64];   Application specific tag field */
        0,         /* RESERVED1;            Reserved for Intel use */
        0,         /* ConnectSeconds;       Time length of connection: seconds */
        0,         /* ConnectMinutes;       Time length of connection: minutes */
        0,         /* ConnectHours;         Time length of connection: hours */
        0,         /* TotalPages;           Total # of pages in all files */
        0,         /* PagesSent;            Number of pages transmitted so far */
        0,         /* FilesSent;            Number of files transmitted so far */
        0,         /* SendCover;            1:Send/0:don't send cover page */
        0,         /* ErrorCount;           Total # of tranmission errors */
        0,         /* RESERVED2[78];        Reserved for Intel use */
        0,         /* RemoteCSID[21];       Remote CCITT id # (If left NULL, the
                                               installed default will be used)*/
        0,         /* DestinationName[32];  To: field */
        0,         /* SenderName[32];       From: field (If left NULL, the
                                               installed default will be used*/
        0          /* LogoFilePath[80];     Path to logo file (If left NULL, the
                                               installed default is used) */
      },
      0,           /*  *CoverPageText;      Variable length  optional cover */
      &DefaultsFTRL   /* *FirstFTR;         Null terminated linked list */
    };
