/* FAXERROR.H  Header file for CAS Toolkit high-level function FAXError.

   Header file that defines the constants used in handling errors in the
   CAS Toolkit high-level functions, and includes a table of CAS error codes
   matched with strings describing them.

   This file must be included in the source files of any application that uses
   the high-level Toolkit function FAXError.
==============================================================================*/
/*
    ERROR MESSAGE LIST for high-level Toolkit functions.  This list is indexed
    by the global variable FAXerrno.
*/
char *FAXerrlist[] = {

  /* First, the names of the CAS low-level functions, by their hex numbers */
  "CASGetInstalledState ",                          /* INSTALLEDSTATE 0x00 */
  "CASSubmitTask ",                                 /* SUBMITTASK     0x01 */
  "CASAbortCurrentEvent ",                          /* ABORTCURRENT   0x02 */
  "",
  "",
  "CASFindFirst ",                                  /* FINDFIRST      0x05 */
  "CASFindNext ",                                   /* FINDNEXT       0x06 */
  "CASOpenFile",                                    /* OPENFILE       0x07 */
  "CASDeleteFile ",                                 /* DELETEFILE     0x08 */
  "CASDeleteAllFiles ",                             /* DELETEALL      0x09 */
  "CASGetEventDate ",                               /* GETEVENTDATE   0x0A */
  "CASSetEventDate ",                               /* SETEVENTDATE   0x0B */
  "CASGetEventTime ",                               /* GETEVENTTIME   0x0C */
  "CASSetEventTime ",                               /* SETEVENTTIME   0x0D */
  "CASGetExternalData ",                            /* GETEDB         0x0E */
  "CASARState ",                                    /* AUTORCVSTATE   0x0F */
  "CASGetEventStatus ",                             /* GETEVENTSTATUS 0x10 */
  "CASGetQueueStatus ",                             /* GETQUEUESTATUS 0x11 */
  "CASGetHardwareStatus ",                          /* GETHWSTATUS    0x12 */
  "CASRunDiagnostics ",                             /* RUNDIAGNOSTICS 0x13 */
  "CASMoveReceivedFile ",                           /* MOVEFILE       0x14 */
  "CASSubmitSingleFile ",                           /* SUBMITSINGLE   0x15 */
  "CASUninstall ",                                  /* UNINSTALL      0x16 */

  /* Toolkit error checking errors */
  "Error using lseek",                                                  /* 23 LSEEKERROR          */
  "Insufficient memory",                                                /* 24 OUTOFMEMORY         */
  "Error opening file",                                                 /* 25 CANTOPENFILE        */
  "Error writing to file",                                              /* 26 CANTWRITEFILE       */
  "Error reading file",                                                 /* 27 CANTREADFILE        */
  "Error closing file",                                                 /* 28 CANTCLOSEFILE       */
  "FileCount field differs from actual # of files",                     /* 29 BADFILECOUNT        */
  "Structure freed pointed to another structure",                       /* 30 ORPHANWARNING       */
  "No Control file for the given event",                                /* 31 EVENTNOTFOUND       */
  "Bad ECS: FileCount is less than 0 or greater than 32766",            /* 32 FILECOUNTOUTOFRANGE */
  "The file number requested was too high",                             /* 33 NOTTHATMANYFILES    */
  "The cancelled event was currently executing",                        /* 34 EVENTWASCURRENT     */
  "Invalid Task EventStatus",                                           /* 35 BADSTATUS           */
  "Caller's file list overwrites existing FTR pointers",                /* 36 LOSTFTRS            */
  "End of file, but there should have been more FTR's",                 /* 37 UNEXPECTEDEOF       */
  "There are more FTR's than FileCount",                                /* 38 MOREFTRSTHANFC      */
  "An ASCIIZ string is longer than allowed",                            /* 39 STRINGTOOLONG       */
  "A pointer was uninitialized (NULL)",                                 /* 40 INVALIDPTR          */
  "Bad Event Control Structure: EventType",                             /* 41 BADEVENTTYPE        */
  "Bad Event Control Structure: TransferType",                          /* 42 BADTRANSFERTYPE     */
  "Warning: A Resident Manager field was not null",                     /* 43 FIELDZEROED         */
  "Bad ECS: EventTime",                                                 /* 44 BADEVENTTIME        */
  "Bad ECS: EventDate",                                                 /* 45 BADEVENTDATE        */
  "Bad ECS: Phonenumber",                                               /* 46 BADPHONENUMBER      */
  "Bad ECS: An ASCII string contained a non-ASCII character",           /* 47 NONASCIISTRING      */
  "Bad ECS: SendCover",                                                 /* 48 BADSENDCOVER        */
  "Warning: A File Transfer Record field was not null",                 /* 49 FTRFIELDZEROED      */
  "Bad FTR: FileType",                                                  /* 50 FTRBADFILETYPE      */
  "Bad FTR: TextSize",                                                  /* 51 FTRBADTEXTSIZE      */
  "Bad FTR: PageLength",                                                /* 52 FTRBADPAGELENGTH    */
  "Bad Ftr: An ASCII string contained a non-ASCII character",           /* 53 FTRNONASCIISTRING   */
  "A Toolkit set a reserved field to all zeroes",                       /* 54 RESERVEDFIELDZEROED */
  "A reserved field in a File Transfer Record was not null",            /* 55 FTRRESERVEDZEROED   */
  "Warning, FTR: The additional page increments value is out-of-range", /* 56 FTRBADPAGEINCREMENTS*/
  "Bad ECS: Logo file or path not found",                               /* 57 LOGOFILENOTFOUND    */
  "Bad FTR: File to send or path to file not found",                    /* 58 FILETOSENDNOTFOUND  */
  "Warning: The defaults Event structure contains some invalid data",   /* 59 INVALIDDEFAULTS     */
  "Warning: The parameter Event structure contains some invalid data",  /* 60 INVALIDECS          */
  "There was no cover text for the event",                              /* 61 NOCOVERTEXT         */
  "Information not available because the event is currently executing", /* 62 EVENTISCURRENT      */
  "Can't retrieve the cover page text of a received fax",               /* 63 RECEIVEDFAXNOCOVER  */
  "Invalid FAXerrno"                                                    /* 64 LASTMESSAGE         */
  };

/*
    CAS ERROR MESSAGE TABLE:  This table maps CAS error codes with CAS error
    classes and appropriate messages.
*/
ErrorTable CASerrors[] = {
    { 0x0000, "" , FAXWARNING},
    { 0x0002, "(0002) Bad scanline count" , FAXWARNING},
    { 0x0003, "(0003) Page sent with errors, could not resend" , FAXWARNING},
    { 0x0004, "(0004) Received data lost" , FAXWARNING},
    { 0x0005, "(0005) Invalid or missing logo file" , FAXWARNING},
    { 0x0006, "(0006) File name does not match NSF header" , FAXWARNING},
    { 0x0007, "(0007) File size does not match NSF header" , FAXWARNING},
    { 0x0101, "(0101) Invalid function number" , DOSWARNING},
    { 0x0105, "(0105) Access denied" , DOSWARNING},
    { 0x0106, "(0106) Invalid handle" , DOSWARNING},
    { 0x0200, "(0200) Multiplex handler failed" , FATALERROR},
    { 0x0201, "(0201) Unknown command (invalid function number)" , FATALERROR},
    { 0x0202, "(0202) Event not found (invlaid event handle)" , FATALERROR},
    { 0x0203, "(0203) Attempted to Find Next before Find First" , FATALERROR},
    { 0x0204, "(0204) No more events" , FATALERROR},
    { 0x0207, "(0207) Invalid Queue type (bad Queue number)" , FATALERROR},
    { 0x0208, "(0208) Bad Control file" , FATALERROR},
    { 0x0209, "(0209) Communication board is busy" , FATALERROR},
    { 0x020A, "(020A) Invalid command parameter" , FATALERROR},
    { 0x020B, "(020B) Can not uninstall Resident Manager" , FATALERROR},
    { 0x020C, "(020C) File already exists" , FATALERROR},
    { 0x0280, "(0280) Invalid task type (not a Send or Poll event" , FATALERROR},
    { 0x0281, "(0281) Bad phone number" , FATALERROR},
    { 0x0282, "(0282) Bad PCX file header" , FATALERROR},
    { 0x0283, "(0283) Unexpected End of File" , FATALERROR},
    { 0x0284, "(0284) Unexpected disconnect" , FATALERROR},
    { 0x0285, "(0285) Exceeded maximum dialing retries" , FATALERROR},
    { 0x0286, "(0286) No files were specified for send event" , FATALERROR},
    { 0x0287, "(0287) Communication board timeout" , FATALERROR},
    { 0x0288, "(0288) Received more than 1023 pages of data" , FATALERROR},
    { 0x0289, "(0289) Manual connect posted too long ago" , FATALERROR},
    { 0x028A, "(028A) Hardware command set error" , FATALERROR},
    { 0x028B, "(028B) Bad nonstandard NSF header file" , FATALERROR},
    { 0x0302, "(0302) File not found" , FATALDOSERROR},
    { 0x0303, "(0303) Path not found" , FATALDOSERROR},
    { 0x0401, "(0401) Remote unit not Group 3 compatible" ,               FAXERROR},
    { 0x0402, "(0402) Remote unit didn't send its capabilities" ,         FAXERROR},
    { 0x0403, "(0403) Remote unit requested disconnect" ,                 FAXERROR},
    { 0x0404, "(0404) Remote unit isn't a Connection Coprocessor" ,       FAXERROR},
    { 0x0405, "(0405) Exceeded retrain or fax resend limit" ,             FAXERROR},
    { 0x0406, "(0406) Line noise or the local and remote unit don't agree on a bit rate",FAXERROR},
    { 0x0407, "(0407) Remote unit disconnected after receiving data" ,    FAXERROR},
    { 0x0408, "(0408) No response from remote unit after sending data" ,  FAXERROR},
    { 0x040A, "(040A) Capabilities of remote unit aren't compatible" ,    FAXERROR},
    { 0x040C, "(040C) Invalid response from remote unit after sending data" ,FAXERROR},
    { 0x0410, "(0410) Phone line dead or remote unit disconnected",       FAXERROR},
    { 0x0414, "(0414) Invalid command from remote after receiving data ", FAXERROR},
    { 0x041E, "(041E) Tried to receive from incompatible hardware",       FAXERROR},
    { 0x045B, "(045B) Unexpected end of file while receiving",            FAXERROR},
    { 0x045C, "(045C) Received data overflowed input buffer",             FAXERROR},
    { 0x045D, "(045D) Remote hardware unexpectedly stopped sending data", FAXERROR},
    { 0x045E, "(045E) Remote hardware didn't send any data",              FAXERROR},
    { 0x0462, "(0462) Remote hardware took too long to send fax scan line",FAXERROR},
    { 0x0463, "(0463) Can't get through to remote unit",                  FAXERROR},
    { 0x0464, "(0464) User canceled event",                               FAXERROR}
};
